/**
 * libSpiff - XSPF playlist handling library
 *
 * Copyright (C) 2006 Sebastian Pipping
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Sebastian Pipping, webmaster@hartwork.org
 */

#include "../Source/Spiff/Spiff.h"
#include "../Source/Spiff/SpiffToolbox.h"
#include <stdio.h>
#include <sstream>
using namespace Spiff;
using namespace Spiff::Toolbox;



int PORT_MAIN() {
	SpiffWriter writer(1); // Version := 1

	// Example playlist props
	SpiffProps props;
	props.setTitle(newAndCopy(_PT("Now playing")));
	props.setAnnotation(newAndCopy(_PT("Well, ...")));
	props.setCreator(newAndCopy(_PT("libSpiff XSPF library")));
	SpiffDateTime dateTime = {2006, 8, 28, 11, 30, 11, 1, 0};
	props.setDate(&dateTime);
	props.setIdentifier(newAndCopy(_PT("xspf:libSpiff:testId")));
	props.setImage(newAndCopy(_PT("http://big/balloons.jpg")));
	props.setInfo(newAndCopy(_PT("http://info/info")));
	props.setLicense(newAndCopy(_PT("http://license/free/")));
	props.setLocation(newAndCopy(_PT("http://playlist/home.xspf")));
	props.setVersion(0);
	props.addAttributionIdentifier(newAndCopy(_PT("some:any/many")));
	props.addAttributionLocation(newAndCopy(_PT("test:pest/west")));
	props.addLink(newAndCopy(_PT("store:url")), newAndCopy(_PT("http://expensive/dot/com")));
	props.addMeta(newAndCopy(_PT("tag:meta:year")), newAndCopy(_PT("2006")));
	SpiffIndentPropsWriter propsWriter(&props);

	// Example track
	SpiffTrack track;
	track.setAlbum(newAndCopy(_PT("Breaking Benjamin")));
	track.setCreator(newAndCopy(_PT("Phobia")));
	track.setTitle(newAndCopy(_PT("Unknown Soldier")));
	track.setTrackNum(10);
	track.addIdentifier(newAndCopy(_PT("id:bb:unknownSoldier")));
	track.addLink(newAndCopy(_PT("http://website")), newAndCopy(_PT("http://website")));
	track.addLocation(newAndCopy(_PT("http://download/mp3/")));
	track.addMeta(newAndCopy(_PT("tag:meta:year")), newAndCopy(_PT("2006")));
	track.setAnnotation(newAndCopy(_PT("Great song")));
	track.setDuration(123000);
	track.setImage(newAndCopy(_PT("http://big/balloons.jpg")));
	track.setInfo(newAndCopy(_PT("http://info/info")));
	SpiffIndentTrackWriter trackWriter;
	trackWriter.setTrack(&track);

	// Add track three times
	writer.addTrack(trackWriter);
	writer.addTrack(trackWriter);

	// Finish with props
	const int res = writer.write(_PT("TEST.xspf"), propsWriter);
	if (res != SPIFF_WRITER_SUCCESS) {
		PORT_PRINTF(_PT("Could not write to file.\n"));
	} else {
		PORT_PRINTF(_PT("Everything fine.\n"));
	}
	return 0;
}
