/**
 * libSpiff - XSPF playlist handling library
 *
 * Copyright (C) 2006 Sebastian Pipping
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Sebastian Pipping, webmaster@hartwork.org
 */

#include "SpiffIndentTrackWriter.h"
#include "SpiffTrack.h"

namespace Spiff {



SpiffIndentTrackWriter::SpiffIndentTrackWriter() : output(NULL), track(NULL) {

}



SpiffIndentTrackWriter::~SpiffIndentTrackWriter() {

}



void SpiffIndentTrackWriter::setTrack(SpiffTrack * track) {
	this->track = track;
}



void SpiffIndentTrackWriter::init(basic_ostringstream<XML_Char> & output, int version) {
	this->output = &output;
	this->version = version;
}



void SpiffIndentTrackWriter::write() {
	writeTrackOpen();
	writeLocations();
	writeIdentifiers();
	writeTitle();
	writeCreator();
	writeAnnotation();
	writeInfo();
	writeImage();
	writeAlbum();
	writeTrackNum();
	writeDuration();
	writeLinks();
	writeMetas();
	writeExtensions();
	writeTrackClose();
}



void SpiffIndentTrackWriter::writeAlbum() {
	const XML_Char * const album = this->track->getAlbum();
	if (album != NULL) {
		(*this->output) << _PT("\t\t\t<album>") << album << _PT("</album>\n");
	}
}



void SpiffIndentTrackWriter::writeImage() {
	const XML_Char * const image = this->track->getImage();
	if (image != NULL) {
		(*this->output) << _PT("\t\t\t<image>") << image << _PT("</image>\n");
	}
}



void SpiffIndentTrackWriter::writeInfo() {
	const XML_Char * const info = this->track->getInfo();
	if (info != NULL) {
		(*this->output) << _PT("\t\t\t<info>") << info << _PT("</info>\n");
	}
}



void SpiffIndentTrackWriter::writeAnnotation() {
	const XML_Char * const annotation = this->track->getAnnotation();
	if (annotation != NULL) {
		(*this->output) << _PT("\t\t\t<annotation>") << annotation << _PT("</annotation>\n");
	}
}



void SpiffIndentTrackWriter::writeCreator() {
	const XML_Char * const creator = this->track->getCreator();
	if (creator != NULL) {
		(*this->output) << _PT("\t\t\t<creator>") << creator << _PT("</creator>\n");
	}
}



void SpiffIndentTrackWriter::writeTitle() {
	const XML_Char * const title = this->track->getTitle();
	if (title != NULL) {
		(*this->output) << _PT("\t\t\t<title>") << title << _PT("</title>\n");
	}
}



void SpiffIndentTrackWriter::writeDuration() {
	const int duration = this->track->getDuration();
	if (duration != -1) {
		(*this->output) << _PT("\t\t\t<duration>") << duration << _PT("</duration>\n");
	}
}



void SpiffIndentTrackWriter::writeTrackNum() {
	const int trackNum = this->track->getTrackNum();
	if (trackNum != -1) {
		(*this->output) << _PT("\t\t\t<trackNum>") << trackNum << _PT("</trackNum>\n");
	}
}



void SpiffIndentTrackWriter::writeExtensions() {
	// Nothing to do
}



void SpiffIndentTrackWriter::writeMetas() {
	const SpiffMetaLinkList * const metas = this->track->getMetas();
	if (metas != NULL) {
		SpiffMetaLinkList::const_iterator iter = metas->begin();
		while (iter != metas->end()) {
			const SpiffMetaLink & entry = *iter;
			(*this->output) << _PT("\t\t\t<meta rel=\"") << entry.first << _PT("\">")
					<< entry.second << _PT("</meta>\n");
			iter++;
		}
	}
}



void SpiffIndentTrackWriter::writeLinks() {
	const SpiffMetaLinkList * const links = this->track->getLinks();
	if (links != NULL) {
		SpiffMetaLinkList::const_iterator iter = links->begin();
		while (iter != links->end()) {
			const SpiffMetaLink & entry = *iter;
			(*this->output) << _PT("\t\t\t<link rel=\"") << entry.first << _PT("\">")
					<< entry.second << _PT("</link>\n");
			iter++;
		}
	}
}



void SpiffIndentTrackWriter::writeLocations() {
	const SpiffStringList * const locations = this->track->getLocations();
	if (locations != NULL) {
		SpiffStringList::const_iterator iter = locations->begin();
		while (iter != locations->end()) {
			(*this->output) << _PT("\t\t\t<location>") << *iter
						<< _PT("</location>\n");
			iter++;
		}
	}
}



void SpiffIndentTrackWriter::writeIdentifiers() {
	const SpiffStringList * const identifiers = this->track->getIdentifiers();
	if (identifiers != NULL) {
		SpiffStringList::const_iterator iter = identifiers->begin();
		while (iter != identifiers->end()) {
			(*this->output) << _PT("\t\t\t<identifier>") << *iter
						<< _PT("</identifier>\n");
			iter++;
		}
	}
}



void SpiffIndentTrackWriter::writeTrackClose() {
	(*this->output) << _PT("\t\t</track>\n");
}



void SpiffIndentTrackWriter::writeTrackOpen() {
	(*this->output) << _PT("\t\t<track>\n");
}



}
