/**
 * libSpiff - XSPF playlist handling library
 *
 * Copyright (C) 2006 Sebastian Pipping
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Sebastian Pipping, webmaster@hartwork.org
 */

#include "SpiffProps.h"
#include "SpiffToolbox.h"
#include <string.h>

namespace Spiff {



SpiffProps::SpiffProps() : location(NULL), info(NULL), title(NULL),
		creator(NULL), annotation(NULL), license(NULL), identifier(NULL),
		image(NULL), attribution(NULL), meta(NULL), link(NULL), date(NULL), version(-1) {

}



SpiffProps::~SpiffProps() {
	if (this->title != NULL) {
		delete [] this->title;
	}
	if (this->creator != NULL) {
		delete [] this->creator;
	}
	if (this->annotation != NULL) {
		delete [] this->annotation;
	}
	if (this->location != NULL) {
		delete [] this->location;
	}
	if (this->identifier != NULL) {
		delete [] this->identifier;
	}
	if (this->image != NULL) {
		delete [] this->image;
	}
	if (this->info != NULL) {
		delete [] this->info;
	}
	if (this->license != NULL) {
		delete [] this->license;
	}
	if (this->attribution != NULL) {
		SpiffAttributionList::const_iterator iter = this->attribution->begin();
		while (iter != this->attribution->end()) {
			delete [] (*iter).second;
			iter++;
		}
		delete this->attribution;
	}
	if (this->meta != NULL) {
		SpiffMetaLinkList::const_iterator iter = this->meta->begin();
		while (iter != this->meta->end()) {
			const SpiffMetaLink & entry = *iter;
			delete [] entry.first;
			delete [] entry.second;
			iter++;
		}
		delete this->meta;
	}
	if (this->link != NULL) {
		SpiffMetaLinkList::const_iterator iter = this->link->begin();
		while (iter != this->link->end()) {
			const SpiffMetaLink & entry = *iter;
			delete [] entry.first;
			delete [] entry.second;
			iter++;
		}
		delete this->link;
	}
	if (this->date != NULL) {
		delete this->date;
	}
}



void SpiffProps::setTitle(const XML_Char * title) {
	Toolbox::deleteNewAndCopy(&this->title, title);
}



void SpiffProps::setCreator(const XML_Char * creator) {
	Toolbox::deleteNewAndCopy(&this->creator, creator);
}



void SpiffProps::setAnnotation(const XML_Char * annotation) {
	Toolbox::deleteNewAndCopy(&this->annotation, annotation);
}



void SpiffProps::setInfo(const XML_Char * info) {
	Toolbox::deleteNewAndCopy(&this->info, info);
}


void SpiffProps::setLocation(const XML_Char * location) {
	Toolbox::deleteNewAndCopy(&this->location, location);
}



void SpiffProps::setIdentifier(const XML_Char * identifier) {
	Toolbox::deleteNewAndCopy(&this->identifier, identifier);
}



void SpiffProps::setImage(const XML_Char * image) {
	Toolbox::deleteNewAndCopy(&this->image, image);
}



void SpiffProps::setLicense(const XML_Char * license) {
	Toolbox::deleteNewAndCopy(&this->license, license);
}



XML_Char * SpiffProps::stealTitle() {
	return Toolbox::getSetNull<XML_Char>(&this->title);
}



XML_Char * SpiffProps::stealCreator() {
	return Toolbox::getSetNull<XML_Char>(&this->creator);
}



XML_Char * SpiffProps::stealAnnotation() {
	return Toolbox::getSetNull<XML_Char>(&this->annotation);
}



XML_Char * SpiffProps::stealInfo() {
	return Toolbox::getSetNull<XML_Char>(&this->info);
}



XML_Char * SpiffProps::stealLocation() {
	return Toolbox::getSetNull<XML_Char>(&this->location);
}



XML_Char * SpiffProps::stealIdentifier() {
	return Toolbox::getSetNull<XML_Char>(&this->identifier);
}



XML_Char * SpiffProps::stealImage() {
	return Toolbox::getSetNull<XML_Char>(&this->image);
}



XML_Char * SpiffProps::stealLicense() {
	return Toolbox::getSetNull<XML_Char>(&this->license);
}



void SpiffProps::addAttribution(const XML_Char * content, bool isIdentifier) {
	if (this->attribution == NULL) {
		this->attribution = new SpiffAttributionList;
	}
	SpiffAttribution entry(isIdentifier, Toolbox::newAndCopy(content));
	this->attribution->push_back(entry);
}



void SpiffProps::addAttributionIdentifier(const XML_Char * content) {
	addAttribution(content, true);
}



void SpiffProps::addAttributionLocation(const XML_Char * content) {
	addAttribution(content, false);
}



SpiffAttributionList * SpiffProps::stealAttribution() {
	return Toolbox::getSetNull<SpiffAttributionList>(&this->attribution);
}



void SpiffProps::addMetaLink(SpiffMetaLinkList ** list, const XML_Char * rel, const XML_Char * content) {
	if (*list == NULL) {
		*list = new SpiffMetaLinkList;
	}
	SpiffMetaLink entry(Toolbox::newAndCopy(rel), Toolbox::newAndCopy(content));
	(*list)->push_back(entry);
}



void SpiffProps::addMeta(const XML_Char * rel, const XML_Char * content) {
	addMetaLink(&this->meta, rel, content);
}


void SpiffProps::addLink(const XML_Char * rel, const XML_Char * content) {
	addMetaLink(&this->link, rel, content);
}



SpiffMetaLinkList * SpiffProps::stealMeta() {
	return Toolbox::getSetNull<SpiffMetaLinkList>(&this->meta);
}



SpiffMetaLinkList * SpiffProps::stealLink() {
	return Toolbox::getSetNull<SpiffMetaLinkList>(&this->link);
}



void SpiffProps::setDate(const SpiffDateTime * date) {
	if (this->date == NULL) {
		this->date = new SpiffDateTime();
	}
	// TODO
	::memcpy(this->date, date, sizeof(SpiffDateTime));
}



SpiffDateTime * SpiffProps::stealDate() {
	return Toolbox::getSetNull<SpiffDateTime>(&this->date);
}



void SpiffProps::setVersion(int version) {
	this->version = version;
}



int SpiffProps::getVersion() {
	return this->version;
}



const XML_Char* SpiffProps::getLicense() {
	return this->license;
}



const XML_Char* SpiffProps::getInfo() {
	return this->info;
}



const XML_Char* SpiffProps::getTitle() {
	return this->title;
}



const XML_Char* SpiffProps::getCreator() {
	return this->creator;
}



const XML_Char* SpiffProps::getAnnotation() {
	return this->annotation;
}



const XML_Char* SpiffProps::getLocation() {
	return this->location;
}


const XML_Char* SpiffProps::getIdentifier() {
	return this->identifier;
}



const XML_Char* SpiffProps::getImage() {
	return this->image;
}



const SpiffAttributionList* SpiffProps::getAttribution() {
	return this->attribution;
}


const SpiffMetaLinkList* SpiffProps::getMeta() {
	return this->meta;
}



const SpiffMetaLinkList* SpiffProps::getLink() {
	return this->link;
}



const SpiffDateTime* SpiffProps::getDate() {
	return this->date;
}



}
