/**
 * libSpiff - XSPF playlist handling library
 *
 * Copyright (C) 2006 Sebastian Pipping
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Sebastian Pipping, webmaster@hartwork.org
 */

#ifndef SPIFF_PROPS_H
#define SPIFF_PROPS_H 1



#include "SpiffDefines.h"
#include <list>
#include <utility>
using namespace std;

namespace Spiff {



class SpiffDateTime {

public:
	int year;
	int month;
	int day;
	int hour;
	int minutes;
	int seconds;

	int distHours;
	int distMinutes;
	
};



typedef pair<bool, XML_Char *> SpiffAttribution;
typedef list<SpiffAttribution> SpiffAttributionList;



class SpiffProps {

private:
	XML_Char * location;
	XML_Char * info;
	XML_Char * title;
	XML_Char * creator;
	XML_Char * annotation;
	XML_Char * license;
	XML_Char * identifier;
	XML_Char * image;
	SpiffAttributionList * attribution;
	SpiffMetaLinkList * meta;
	SpiffMetaLinkList * link;
	SpiffDateTime * date;
	int version;

public:
	SpiffProps();
	~SpiffProps();

	void setTitle(const XML_Char * title);
	void setCreator(const XML_Char * creator);
	void setAnnotation(const XML_Char * annotation);
	void setInfo(const XML_Char * info);
	void setIdentifier(const XML_Char * identifier);
	void setImage(const XML_Char * image);
	void setLicense(const XML_Char * license);
	void setLocation(const XML_Char * location);
	void addAttributionLocation(const XML_Char * content);
	void addAttributionIdentifier(const XML_Char * content);
	void addMeta(const XML_Char * rel, const XML_Char * content);
	void addLink(const XML_Char * rel, const XML_Char * content);
	void setDate(const SpiffDateTime * date);

	XML_Char * stealLicense();
	XML_Char * stealInfo();
	XML_Char * stealTitle();
	XML_Char * stealCreator();
	XML_Char * stealAnnotation();
	XML_Char * stealLocation();
	XML_Char * stealIdentifier();
	XML_Char * stealImage();
	SpiffAttributionList * stealAttribution();
	SpiffMetaLinkList * stealMeta();
	SpiffMetaLinkList * stealLink();
	SpiffDateTime * stealDate();

	const XML_Char * getLicense();
	const XML_Char * getInfo();
	const XML_Char * getTitle();
	const XML_Char * getCreator();
	const XML_Char * getAnnotation();
	const XML_Char * getLocation();
	const XML_Char * getIdentifier();
	const XML_Char * getImage();
	const SpiffAttributionList * getAttribution();
	const SpiffMetaLinkList * getMeta();
	const SpiffMetaLinkList * getLink();
	const SpiffDateTime * getDate();

	void setVersion(int version);
	int getVersion();

private:
	void addAttribution(const XML_Char * content, bool isIdentifier);
	void addMetaLink(SpiffMetaLinkList ** list, const XML_Char * rel, const XML_Char * content);

};



}

#endif // SPIFF_PROPS_H
