/**
 * libSpiff - XSPF playlist handling library
 *
 * Copyright (C) 2006 Sebastian Pipping
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Sebastian Pipping, webmaster@hartwork.org
 */

#ifndef SPIFF_TRACK_H
#define SPIFF_TRACK_H 1



#include "SpiffDefines.h"
#include <list>
#include <utility>
using namespace std;

namespace Spiff {



class SpiffTrack {

private:
	XML_Char * album;
	XML_Char * image;
	XML_Char * info;
	XML_Char * annotation;
	XML_Char * creator;
	XML_Char * title;
	SpiffStringList * locations;
	SpiffStringList * identifiers;
	SpiffMetaLinkList * links;
	SpiffMetaLinkList * metas;
	int trackNum; // > 0
	int duration; // Milliseconds

public:
	SpiffTrack();
	~SpiffTrack();

	void setImage(const XML_Char * image);
	void setInfo(const XML_Char * info);
	void setAnnotation(const XML_Char * annotation);
	void setCreator(const XML_Char * creator);
	void setTitle(const XML_Char * title);
	void setAlbum(const XML_Char * album);
	void setTrackNum(int trackNum);
	void setDuration(int duration);
	void addIdentifier(const XML_Char * identifier);
	void addLocation(const XML_Char * location);
	void addLink(const XML_Char * rel, const XML_Char * content);
	void addMeta(const XML_Char * rel, const XML_Char * content);

	XML_Char * stealAlbum();
	XML_Char * stealImage();
	XML_Char * stealInfo();
	XML_Char * stealAnnotation();
	XML_Char * stealCreator();
	XML_Char * stealTitle();
	SpiffStringList * stealIdentifier();
	SpiffStringList * stealLocation();
	SpiffMetaLinkList * stealLink();
	SpiffMetaLinkList * stealMeta();

	const XML_Char * getAlbum();
	const XML_Char * getImage();
	const XML_Char * getInfo();
	const XML_Char * getAnnotation();
	const XML_Char * getCreator();
	const XML_Char * getTitle();
	const SpiffStringList * getLocations();
	const SpiffStringList * getIdentifiers();
	const SpiffMetaLinkList * getLinks();
	const SpiffMetaLinkList * getMetas();

	int getDuration();
	int getTrackNum();

};



}

#endif // SPIFF_TRACK_H
