/**
 * libSpiff - XSPF playlist handling library
 *
 * Copyright (C) 2006 Sebastian Pipping
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Sebastian Pipping, webmaster@hartwork.org
 */

#include "../CppTest/cpptest.h"
#include "../Spiff/Spiff.h"
using namespace Test;
using namespace Spiff;



#define FAIL_PATH _PT(".\\fail\\")
#define PASS_PATH _PT(".\\pass\\")



class SpiffSuite : public Suite {

private:
	SpiffReader parser;

public:
	SpiffSuite() {
		// Fail
		TEST_ADD(SpiffSuite::libspiff_playlist_annotation_attribute_forbidden)
		TEST_ADD(SpiffSuite::libspiff_playlist_attribution_element_forbidden)
		TEST_ADD(SpiffSuite::libspiff_playlist_extension_in_v0)
		TEST_ADD(SpiffSuite::libspiff_root_name)
		TEST_ADD(SpiffSuite::libspiff_track_extension_in_v0)
		TEST_ADD(SpiffSuite::libspiff_tracklist_empty_in_v0)
		TEST_ADD(SpiffSuite::libspiff_xml_error)
			
		// Pass
		TEST_ADD(SpiffSuite::libspiff_playlist_extension)
		TEST_ADD(SpiffSuite::libspiff_track_extension)

		// Fail
		TEST_ADD(SpiffSuite::playlist_baddate)
		TEST_ADD(SpiffSuite::playlist_badversion)
		TEST_ADD(SpiffSuite::playlist_containshtml_annotation)
		TEST_ADD(SpiffSuite::playlist_containshtml_creator)
		TEST_ADD(SpiffSuite::playlist_containshtml_title)
		TEST_ADD(SpiffSuite::playlist_invalidnamespace)
		TEST_ADD(SpiffSuite::playlist_missingnamespace)
		TEST_ADD(SpiffSuite::playlist_missingtracklist)
		TEST_ADD(SpiffSuite::playlist_missingversion)
		TEST_ADD(SpiffSuite::playlist_noturi_attribution_identifier)
		TEST_ADD(SpiffSuite::playlist_noturi_attribution_location)
		TEST_ADD(SpiffSuite::playlist_noturi_extension)
		TEST_ADD(SpiffSuite::playlist_noturi_identifier)
		TEST_ADD(SpiffSuite::playlist_noturi_image)
		TEST_ADD(SpiffSuite::playlist_noturi_info)
		TEST_ADD(SpiffSuite::playlist_noturi_license)
		TEST_ADD(SpiffSuite::playlist_noturi_link_content)
		TEST_ADD(SpiffSuite::playlist_noturi_link_rel)
		TEST_ADD(SpiffSuite::playlist_noturi_location)
		TEST_ADD(SpiffSuite::playlist_noturi_meta)
		TEST_ADD(SpiffSuite::playlist_toomany_annotation)
		TEST_ADD(SpiffSuite::playlist_toomany_attribution)
		TEST_ADD(SpiffSuite::playlist_toomany_creator)
		TEST_ADD(SpiffSuite::playlist_toomany_date)
		TEST_ADD(SpiffSuite::playlist_toomany_identifier)
		TEST_ADD(SpiffSuite::playlist_toomany_image)
		TEST_ADD(SpiffSuite::playlist_toomany_info)
		TEST_ADD(SpiffSuite::playlist_toomany_license)
		TEST_ADD(SpiffSuite::playlist_toomany_location)
		TEST_ADD(SpiffSuite::playlist_toomany_title)
		TEST_ADD(SpiffSuite::playlist_toomany_tracklist)
		TEST_ADD(SpiffSuite::track_badint_duration)
		TEST_ADD(SpiffSuite::track_badint_tracknum)
		TEST_ADD(SpiffSuite::track_markup_album)
		TEST_ADD(SpiffSuite::track_markup_annotation)
		TEST_ADD(SpiffSuite::track_markup_creator)
		TEST_ADD(SpiffSuite::track_markup_title)
		TEST_ADD(SpiffSuite::track_noturi_extension)
		TEST_ADD(SpiffSuite::track_noturi_identifier)
		TEST_ADD(SpiffSuite::track_noturi_image)
		TEST_ADD(SpiffSuite::track_noturi_info)
		TEST_ADD(SpiffSuite::track_noturi_link)
		TEST_ADD(SpiffSuite::track_noturi_link_rel)
		TEST_ADD(SpiffSuite::track_noturi_location)
		TEST_ADD(SpiffSuite::track_noturi_meta_rel)
		TEST_ADD(SpiffSuite::track_toomany_album)
		TEST_ADD(SpiffSuite::track_toomany_annotation)
		TEST_ADD(SpiffSuite::track_toomany_creator)
		TEST_ADD(SpiffSuite::track_toomany_duration)
		TEST_ADD(SpiffSuite::track_toomany_image)
		TEST_ADD(SpiffSuite::track_toomany_info)
		TEST_ADD(SpiffSuite::track_toomany_title)
		TEST_ADD(SpiffSuite::track_toomany_tracknum)

		// Pass
		TEST_ADD(SpiffSuite::playlist_extensive)
		TEST_ADD(SpiffSuite::track_extensive)

		// Mixed
		TEST_ADD(SpiffSuite::uri)
		TEST_ADD(SpiffSuite::date)
	}

private:
	void runCase(const XML_Char * filename, int expectedCode) {
		const int res = this->parser.parse(filename, NULL);
		if (res != expectedCode) {
			PORT_PRINTF(
					_PT("      File : %s\n")
					_PT("      Line : %i\n")
					_PT("Error text : '%s'\n")
					_PT("Error code : %i\n")
					_PT("  Expected : %i\n\n"), filename, parser.getErrorLine(),
					parser.getErrorText(), res, expectedCode);
		}
		TEST_ASSERT(res == expectedCode);
	}

	void libspiff_playlist_annotation_attribute_forbidden() {
		runCase(FAIL_PATH _PT("libspiff_playlist_annotation_attribute_forbidden.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_FORBIDDEN);
	}

	void libspiff_playlist_attribution_element_forbidden() {
		runCase(FAIL_PATH _PT("libspiff_playlist_attribution_element_forbidden.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	}

	void libspiff_playlist_extension_in_v0() {
		runCase(FAIL_PATH _PT("libspiff_playlist_extension_in_v0.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	}

	void libspiff_root_name() {
		runCase(FAIL_PATH _PT("libspiff_root_name.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	}

	void libspiff_track_extension_in_v0() {
		runCase(FAIL_PATH _PT("libspiff_track_extension_in_v0.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	}

	void libspiff_tracklist_empty_in_v0() {
		runCase(FAIL_PATH _PT("libspiff_tracklist_empty_in_v0.xspf"), SPIFF_PARSER_ERROR_ELEMENT_MISSING);
	}

	void libspiff_xml_error() {
		runCase(FAIL_PATH _PT("libspiff_xml_error.xspf"), SPIFF_PARSER_ERROR_EXPAT + XML_ERROR_DUPLICATE_ATTRIBUTE);
	}

	void libspiff_playlist_extension() {
		runCase(PASS_PATH _PT("libspiff_playlist_extension.xspf"), SPIFF_PARSER_SUCCESS);
	}

	void libspiff_track_extension() {
		runCase(PASS_PATH _PT("libspiff_track_extension.xspf"), SPIFF_PARSER_SUCCESS);
	}

	void playlist_baddate() {
		runCase(FAIL_PATH _PT("playlist_baddate.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_badversion() {
		runCase(FAIL_PATH _PT("playlist_badversion.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_INVALID);
	};

	void playlist_containshtml_annotation() {
		runCase(FAIL_PATH _PT("playlist_containshtml_annotation.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	};

	void playlist_containshtml_creator() {
		runCase(FAIL_PATH _PT("playlist_containshtml_creator.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	};

	void playlist_containshtml_title() {
		runCase(FAIL_PATH _PT("playlist_containshtml_title.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	};

	void playlist_invalidnamespace() {
		runCase(FAIL_PATH _PT("playlist_invalidnamespace.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_INVALID);
	};

	void playlist_missingnamespace() {
		runCase(FAIL_PATH _PT("playlist_missingnamespace.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_MISSING);
	};

	void playlist_missingtracklist() {
		runCase(FAIL_PATH _PT("playlist_missingtracklist.xspf"), SPIFF_PARSER_ERROR_ELEMENT_MISSING);
	};

	void playlist_missingversion() {
		runCase(FAIL_PATH _PT("playlist_missingversion.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_MISSING);
	};

	void playlist_noturi_attribution_identifier() {
		runCase(FAIL_PATH _PT("playlist_noturi_attribution_identifier.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_noturi_attribution_location() {
		runCase(FAIL_PATH _PT("playlist_noturi_attribution_location.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_noturi_extension() {
		runCase(FAIL_PATH _PT("playlist_noturi_extension.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_INVALID);
	};

	void playlist_noturi_identifier() {
		runCase(FAIL_PATH _PT("playlist_noturi_identifier.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_noturi_image() {
		runCase(FAIL_PATH _PT("playlist_noturi_image.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_noturi_info() {
		runCase(FAIL_PATH _PT("playlist_noturi_info.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_noturi_license() {
		runCase(FAIL_PATH _PT("playlist_noturi_license.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_noturi_link_content() {
		runCase(FAIL_PATH _PT("playlist_noturi_link_content.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_noturi_link_rel() {
		runCase(FAIL_PATH _PT("playlist_noturi_link_rel.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_INVALID);
	};

	void playlist_noturi_location() {
		runCase(FAIL_PATH _PT("playlist_noturi_location.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void playlist_noturi_meta() {
		runCase(FAIL_PATH _PT("playlist_noturi_meta.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_INVALID);
	};

	void playlist_toomany_annotation() {
		runCase(FAIL_PATH _PT("playlist_toomany_annotation.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_attribution() {
		runCase(FAIL_PATH _PT("playlist_toomany_attribution.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_creator() {
		runCase(FAIL_PATH _PT("playlist_toomany_creator.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_date() {
		runCase(FAIL_PATH _PT("playlist_toomany_date.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_identifier() {
		runCase(FAIL_PATH _PT("playlist_toomany_identifier.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_image() {
		runCase(FAIL_PATH _PT("playlist_toomany_image.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_info() {
		runCase(FAIL_PATH _PT("playlist_toomany_info.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_license() {
		runCase(FAIL_PATH _PT("playlist_toomany_license.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_location() {
		runCase(FAIL_PATH _PT("playlist_toomany_location.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_title() {
		runCase(FAIL_PATH _PT("playlist_toomany_title.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void playlist_toomany_tracklist() {
		runCase(FAIL_PATH _PT("playlist_toomany_tracklist.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void track_badint_duration() {
		runCase(FAIL_PATH _PT("track_badint_duration.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void track_badint_tracknum() {
		runCase(FAIL_PATH _PT("track_badint_tracknum.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void track_markup_album() {
		runCase(FAIL_PATH _PT("track_markup_album.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	};

	void track_markup_annotation() {
		runCase(FAIL_PATH _PT("track_markup_annotation.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	};

	void track_markup_creator() {
		runCase(FAIL_PATH _PT("track_markup_creator.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	};

	void track_markup_title() {
		runCase(FAIL_PATH _PT("track_markup_title.xspf"), SPIFF_PARSER_ERROR_ELEMENT_FORBIDDEN);
	};

	void track_noturi_extension() {
		runCase(FAIL_PATH _PT("track_noturi_extension.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_INVALID);
	};

	void track_noturi_identifier() {
		runCase(FAIL_PATH _PT("track_noturi_identifier.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void track_noturi_image() {
		runCase(FAIL_PATH _PT("track_noturi_image.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void track_noturi_info() {
		runCase(FAIL_PATH _PT("track_noturi_info.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void track_noturi_link() {
		runCase(FAIL_PATH _PT("track_noturi_link.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void track_noturi_link_rel() {
		runCase(FAIL_PATH _PT("track_noturi_link_rel.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_INVALID);
	};

	void track_noturi_location() {
		runCase(FAIL_PATH _PT("track_noturi_location.xspf"), SPIFF_PARSER_ERROR_CONTENT_INVALID);
	};

	void track_noturi_meta_rel() {
		runCase(FAIL_PATH _PT("track_noturi_meta_rel.xspf"), SPIFF_PARSER_ERROR_ATTRIBUTE_INVALID);
	};

	void track_toomany_album() {
		runCase(FAIL_PATH _PT("track_toomany_album.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void track_toomany_annotation() {
		runCase(FAIL_PATH _PT("track_toomany_annotation.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void track_toomany_creator() {
		runCase(FAIL_PATH _PT("track_toomany_creator.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void track_toomany_duration() {
		runCase(FAIL_PATH _PT("track_toomany_duration.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void track_toomany_image() {
		runCase(FAIL_PATH _PT("track_toomany_image.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void track_toomany_info() {
		runCase(FAIL_PATH _PT("track_toomany_info.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void track_toomany_title() {
		runCase(FAIL_PATH _PT("track_toomany_title.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};

	void track_toomany_tracknum() {
		runCase(FAIL_PATH _PT("track_toomany_tracknum.xspf"), SPIFF_PARSER_ERROR_ELEMENT_TOOMANY);
	};


	void playlist_extensive() {
		runCase(PASS_PATH _PT("playlist_extensive.xspf"), SPIFF_PARSER_SUCCESS);
	};

	void track_extensive() {
		runCase(PASS_PATH _PT("track_extensive.xspf"), SPIFF_PARSER_SUCCESS);
	};

	void uri() {
		TEST_ASSERT(!SpiffReader::isURI(_PT("abc")));
		TEST_ASSERT(SpiffReader::isURI(_PT("ldap://[2001:db8::7]/c=GB?objectClass?one")));
		TEST_ASSERT(SpiffReader::isURI(_PT("tel:+1-816-555-1212")));
		TEST_ASSERT(SpiffReader::isURI(_PT("urn:oasis:names:specification:docbook:dtd:xml:4.1.2")));
		TEST_ASSERT(SpiffReader::isURI(_PT("file:///F:/abc.txt")));
		TEST_ASSERT(SpiffReader::isURI(_PT("ftp://ftp.is.co.za/rfc/rfc1808.txt")));
		TEST_ASSERT(SpiffReader::isURI(_PT("http://www.ietf.org/rfc/rfc2396.txt")));
		TEST_ASSERT(SpiffReader::isURI(_PT("mailto:John.Doe@example.com")));
		TEST_ASSERT(SpiffReader::isURI(_PT("news:comp.infosystems.www.servers.unix")));
		TEST_ASSERT(SpiffReader::isURI(_PT("telnet://192.0.2.16:80/")));
	}

	void date() {
		SpiffDateTime dateTime;
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("abc"), &dateTime));
		TEST_ASSERT(SpiffReader::extractDateTime(_PT("-2006-08-27T20:34:22.999+01:00"), &dateTime));

		TEST_ASSERT(SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00.000000001-13:59"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00.000000000-13:59"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00.-13:59"), &dateTime));

		TEST_ASSERT(SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00-13:59"), &dateTime));
		TEST_ASSERT(SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00-14:00"), &dateTime));
		TEST_ASSERT(SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00Z"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00z"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01t00:00:00"), &dateTime));
		TEST_ASSERT(SpiffReader::extractDateTime(_PT("9999-12-31T23:59:59+13:59"), &dateTime));
		TEST_ASSERT(SpiffReader::extractDateTime(_PT("9999-12-31T23:59:59+14:00"), &dateTime));

		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0000-01-01T00:00:00-13:59"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-00-01T00:00:00-13:59"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-00T00:00:00-13:59"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01T24:00:00-13:59"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01T00:60:00-13:59"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01T00:00:60-13:59"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00-14:01"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("0001-01-01T00:00:00+14:01"), &dateTime));

		// Leap years
		TEST_ASSERT(SpiffReader::extractDateTime(_PT("1600-02-29T00:00:00"), &dateTime));
		TEST_ASSERT(SpiffReader::extractDateTime(_PT("2004-02-29T00:00:00"), &dateTime));
		TEST_ASSERT(!SpiffReader::extractDateTime(_PT("1700-02-29T00:00:00"), &dateTime));
	}

};



int PORT_MAIN() {
	SpiffSuite suite;
    TextOutput output(TextOutput::Verbose);
	suite.run(output, false);
    return 0;
}
